.\" ====================================================================
.\"  @Troff-man-file{
.\"     author          = "Nelson H. F. Beebe",
.\"     version         = "1.00",
.\"     date            = "29 March 1999",
.\"     time            = "19:04:14 MST",
.\"     filename        = "awklex.man",
.\"     address         = "Center for Scientific Computing
.\"                        University of Utah
.\"                        Department of Mathematics, 322 INSCC
.\"                        155 S 1400 E RM 233
.\"                        Salt Lake City, UT 84112-0090
.\"                        USA",
.\"     telephone       = "+1 801 581 5254",
.\"     FAX             = "+1 801 585 1640, +1 801 581 4148",
.\"     URL             = "http://www.math.utah.edu/~beebe",
.\"     checksum        = "50360 558 2659 18200",
.\"     email           = "beebe@math.utah.edu, beebe@acm.org,
.\"                        beebe@ieee.org (Internet)",
.\"     codetable       = "ISO/ASCII",
.\"     keywords        = "awk, lexer, lexical analyzer",
.\"     supported       = "yes",
.\"     docstring       = "This file contains the UNIX manual pages
.\"                        for awklex, a lexical analyzer for awk
.\"                        program files.
.\"
.\"                        The checksum field above contains a CRC-16
.\"                        checksum as the first value, followed by the
.\"                        equivalent of the standard UNIX wc (word
.\"                        count) utility output of lines, words, and
.\"                        characters.  This is produced by Robert
.\"                        Solovay's checksum utility.",
.\"  }
.\" ====================================================================
.TH AWKLEX 1 "29 March 1999" "Version 1.00"
.\"======================================================================
.SH NAME
awklex \- lexically analyze awk program files
.\"=====================================================================
.SH SYNOPSIS
.B awklex
.BI \-f " awk-program-file"
.I ">outfile"
.\"=====================================================================
.SH DESCRIPTION
.B awklex
lexically analyzes an
.BR awk (1)
program file, and produces on
.I stdout
a token stream, one line per token.  The output
stream can then be further filtered by other
programs, such as the one embedded in
.BR awkpretty (1).
.PP
The companion program,
.BR awkunlex (1),
turns its input lexical token stream back into an
.BR awk (1)
program.
.PP
Typical uses of these tools look like this:
.RS
.nf
\&\fCawklex \fIfile(s)\fP | \fIcmds\fP
\&\fIcmds\fP | awklex -f - | \fIcmds\fP
awklex \fIfile(s)\fP | \fIcmds\fP | awkunlex >new-awk-file
\&\fIcmds\fP | awklex -f - | \fIcmds\fP | awkunlex >new-awk-file\fP
.fi
.RE
.\"=====================================================================
.SH OPTIONS
Although
.B awklex
accepts all of the command-line options recognized
by
.BR awk (1),
the only relevant option is:
.\"-----------------------------------------------
.TP \w'\-f-awk-program-file'u+3n
.BI \-f " awk-program-file"
Specify the name of the file containing the
.BR awk (1)
program.
.IP
Multiple
.B \-f
options can be used, if required; their files are
then treated as if logically concatenated in the
order that they were specified.
.IP
The filename can be a hyphen, \-, meaning that
input should be taken from
.IR stdin ,
allowing
.B awklex
to be used in the middle of a command pipeline.
.\"=====================================================================
.SH "LEXICAL ANALYSIS"
The output stream produced by
.B awklex
has lines of the form
.PP
.RS
.nf
\&\fC# line nnn "filename"
# arbitrary comment text
<token-number><tab>STRING<tab>"<token-value>"
<token-number><tab><token-type-name><tab><token-value>\fP
.fi
.RE
.PP
Each output line is either a line number directive
identifying the source of the original tokens, or
a comment, or else contains a single complete
token, identified by an integer number for use by
a computer program, a token type name for human
readers (and programs), and a token value.
.PP
Errors detected during the lexical and grammatical
analyses produce messages on
.IR stderr ,
but unless a catastrophic error has occurred,
processing will normally continue until all input
has been processed, and it will not be evident
from the token stream that errors have occurred.
Therefore, in order to permit subsequent
applications to detect such errors, and take
suitable recovery action, a special error comment
directive
.RS
.nf
\&\fC# ERROR\fP
.fi
.RE
is produced immediately following the line for the
token at which the error was first reported.
.PP
If the type name is STRING, the value
.I includes
the surrounding quotes.  Special characters in the
token value string are represented with ANSI/ISO
Standard C character and octal escape sequences,
so all characters other than NUL are
representable, and multi-line values can be
represented in a single line.  Tab characters will
not appear literally: they will always be
represented by the escape sequence \fC\et\fP.
.PP
It is possible for a non-STRING value to contain
literal tab characters, so programs that process
the
.B awklex
output stream must be prepared to handle that.
.PP
The special case of a newline (NL) token has an
empty value string, rather than a literal newline
character.
.PP
Except for token types NL and STRING, all other
token values are represented literally; they are
.I not
converted to use Standard C escape sequences, and
cannot contain newline characters.  They extend
from immediately after the second tab character on
the line, up to, but not including, the following
newline character.
.PP
Except for
.BR gawk (1),
most
.BR awk (1)
implementations do not properly handle NUL
characters, and thus,
.B awklex
does not either.  This is a consequence of their
implementation in the C programming language using
NUL-terminated C character strings to hold
.BR awk (1)
strings.  Text files, by definition, do not
contain NUL characters, so this restriction is
seldom of any consequence.
.PP
Here are the token numbers and token type names
that can appear in the output, ordered first by
increasing token number:
.PP
.RS
.nf
\&\fC258 PROGRAM    281 EQ	      304 NEXT	     327 POSTDECR
259 PASTAT     282 GE	      305 NEXTFILE   328 PREDECR
260 PASTAT2    283 GT	      306 ADD	     329 VAR
261 XBEGIN     284 LE	      307 MINUS	     330 IVAR
262 XEND       285 LT	      308 MULT	     331 VARNF
263 NL	       286 NE	      309 DIVIDE     332 CALL
264 ARRAY      287 IN	      310 MOD	     333 NUMBER
265 MATCH      288 ARG	      311 ASSIGN     334 STRING
266 NOTMATCH   289 BLTIN      312 ASGNOP     335 REGEXPR
267 MATCHOP    290 BREAK      313 ADDEQ	     336 COMMENT
268 FINAL      291 CLOSE      314 SUBEQ	     337 WHITESPACE
269 DOT	       292 CONTINUE   315 MULTEQ     338 GETLINE
270 ALL	       293 DELETE     316 DIVEQ	     339 RETURN
271 CCL	       294 DO	      317 MODEQ	     340 SPLIT
272 NCCL       295 EXIT	      318 POWEQ	     341 SUBSTR
273 CHAR       296 FOR	      319 PRINT	     342 WHILE
274 OR	       297 FUNC	      320 PRINTF     343 CAT
275 STAR       298 SUB	      321 SPRINTF    344 NOT
276 QUEST      299 GSUB	      322 ELSE	     345 UMINUS
277 PLUS       300 IF	      323 INTEST     346 POWER
278 AND	       301 INDEX      324 CONDEXPR   347 DECR
279 BOR	       302 LSUBSTR    325 POSTINCR   348 INCR
280 APPEND     303 MATCHFCN   326 PREINCR    349 INDIRECT\fP
.fi
.RE
and then alphabetically by token type name:
.RS
.nf
\&\fC306 ADD	       309 DIVIDE     265 MATCH	     346 POWER
313 ADDEQ      294 DO	      303 MATCHFCN   328 PREDECR
270 ALL	       269 DOT	      267 MATCHOP    326 PREINCR
278 AND	       322 ELSE	      307 MINUS	     319 PRINT
280 APPEND     281 EQ	      310 MOD	     320 PRINTF
288 ARG	       295 EXIT	      317 MODEQ	     258 PROGRAM
264 ARRAY      268 FINAL      308 MULT	     276 QUEST
312 ASGNOP     296 FOR	      315 MULTEQ     335 REGEXPR
311 ASSIGN     297 FUNC	      272 NCCL	     339 RETURN
289 BLTIN      282 GE	      286 NE	     340 SPLIT
279 BOR	       338 GETLINE    304 NEXT	     321 SPRINTF
290 BREAK      299 GSUB	      305 NEXTFILE   275 STAR
332 CALL       283 GT	      263 NL	     334 STRING
343 CAT	       300 IF	      344 NOT	     298 SUB
271 CCL	       287 IN	      266 NOTMATCH   314 SUBEQ
273 CHAR       348 INCR	      333 NUMBER     341 SUBSTR
291 CLOSE      301 INDEX      274 OR	     345 UMINUS
336 COMMENT    349 INDIRECT   259 PASTAT     329 VAR
324 CONDEXPR   323 INTEST     260 PASTAT2    331 VARNF
292 CONTINUE   330 IVAR	      277 PLUS	     342 WHILE
347 DECR       284 LE	      327 POSTDECR   337 WHITESPACE
293 DELETE     302 LSUBSTR    325 POSTINCR   261 XBEGIN
316 DIVEQ      285 LT	      318 POWEQ	     262 XEND\fP
.fi
.RE
.PP
In addition to these 92 uppercase names, token
type names can also be of the form \fCtoken
nnn\fP, where \fCnnn\fP is the decimal ASCII value
of a single-character token, separated by a single
space from the preceding word.  The token number
is also \fCnnn\fP, which is why token numbers 0
\&.\|.\|. 255 were unavailable for the named tokens
listed above.
.PP
The token stream ends with a null token:
.RS
.nf
\&\fC  0        token 0 \fP
.fi
.RE
.PP
The token numbers are generated automatically, and
although
.BR awk (1)
is quite a stable program, it is possible that
they could change in future
.BR awk (1)
versions, so programmers, beware.  The token
.I "type names"
should never change, but the set of token types
may grow if the
.BR awk (1)
language is ever changed.
.PP
The following short example gives a flavor of what
the lexical token stream looks like for this
simple
.BR awk (1)
program:
.RS
.nf
\&\fCBEGIN   { initialize() }
        { print FNR ":" $0 }
END     { terminate() }

function initialize()
{
    print "Hello, world"
}

function terminate()
{
    print "Goodbye, world"
}\fP
.fi
.RE
.PP
Here is the output of \fCawklex -f simple.awk\fP,
displayed in two columns for compactness:
.RS
.nf
\&\fC# line 1 "simple.awk"	      337     WHITESPACE
261	XBEGIN	BEGIN	      332     CALL    initialize
337	WHITESPACE	       40     token 40	      (
123	token 123	{      41     token 41	      )
337	WHITESPACE	      263     NL
332	CALL	initialize    # line 6 "simple.awk"
 40	token 40	(     123     token 123	      {
 41	token 41	)     263     NL
337	WHITESPACE	      # line 7 "simple.awk"
 59	token 59	}     337     WHITESPACE
125	token 125	}     319     PRINT   print
263	NL		      337     WHITESPACE
# line 2 "simple.awk"	      334     STRING  "Hello, world
337	WHITESPACE	      337     WHITESPACE
123	token 123	{     263     NL
337	WHITESPACE	      # line 8 "simple.awk"
319	PRINT	print	       59     token 59	      }
337	WHITESPACE	      125     token 125	      }
329	VAR	FNR	      263     NL
337	WHITESPACE	      # line 10 "simple.awk"
334	STRING	": "	      263     NL
337	WHITESPACE	      # line 10 "simple.awk"
349	INDIRECT	0     297     FUNC    function
333	NUMBER	0	      337     WHITESPACE
337	WHITESPACE	      332     CALL    terminate
 59	token 59	}      40     token 40	      (
125	token 125	}      41     token 41	      )
263	NL		      263     NL
# line 3 "simple.awk"	      # line 11 "simple.awk"
262	XEND	END	      123     token 123	      {
337	WHITESPACE	      263     NL
123	token 123	{     # line 12 "simple.awk"
337	WHITESPACE	      337     WHITESPACE
332	CALL	terminate     319     PRINT   print
 40	token 40	(     337     WHITESPACE
 41	token 41	)     334     STRING  "Goodbye, wor
337	WHITESPACE	      337     WHITESPACE
 59	token 59	}     263     NL
125	token 125	}     # line 13 "simple.awk"
263	NL		       59     token 59	      }
# line 5 "simple.awk"	      125     token 125	      }
263	NL		      263     NL
# line 5 "simple.awk"	      	0     token 0
297	FUNC	function\fP
.fi
.RE
.\"=====================================================================
.SH "SAMPLE APPLICATIONS"
.PP
Here are some possibly useful things you can do
simply with
.BR awklex :
.TP
.I "check spelling:"
\&\fCawklex -f myfile.awk | grep STRING | spell\fP
.TP
.I "check for doubled words:"
\&\fCawklex -f myfile.awk | grep STRING | dw\fP
.TP
.I "count static references to each function:"
\&\fCawklex -f myfile.awk | grep CALL | sort | uniq -c\fP
.TP
.I "find functions that are defined, but never called:"
.nf
\&\fCawklex -f myfile.awk | grep CALL | sort | uniq -c | \e
        awk '$1 == 1' \fP
.fi
.TP
.I "capitalize function names:"
.nf
\&\fCawklex -f myfile.awk | \e
awk '$2 == "CALL"  {printf("%s\et%s\et%s\en", $1, $2, \e
        toupper(substr($3,1,1)) tolower(substr($3,2)))}' | \e
        awkunlex > newfile.awk\fP
.fi
.PP
Because
.B awk
was designed for writing simple programs simply,
it does not require variable declarations, and
except for function arguments, variables are
normally global throughout the entire program.
.PP
The standard idiom for declaring a local variable
in a function is to give it as an extra function
argument.  Failure to do this in a large program
could result in a local variable overwriting a
global variable of the same name, producing a
nasty bug that can be quite hard to detect by
reading the code.
.BR awk (1)
itself cannot diagnose the problem, since it is
legal to do this in the language.
.PP
Here is a how you can find such problems, using
just fifteen lines of
.B awklex
and
.BR awk (1):
.TP
.I "find global variables in functions:"
.RS
.nf
\&\fCawklex -f myfile.awk | \e
awk '
BEGIN       { FS = "\et"; fcn = "-OUTER-" }

($2 == "FUNC"),($3 == ")") { if ($2 == "CALL") fcn = $3; next }

$2 == "VAR" { GlobalVars[fcn,$3]++ }

END         {
                for (fv in GlobalVars)
                {
                    split(fv,vars,SUBSEP)
                    printf("%-31s\et%s\en", vars[1], vars[2]) | "sort"
                }
            }'\fP
.fi
.RE
.PP
This program collects the names of all functions,
and global variables referenced inside them, and
uses them as index pairs in \fCGlobalVars[]\fP to
count the number of such pairs.  After all of the
input stream has been processed, a sorted list is
printed to display the pairs.
.PP
The \fCBEGIN\fP pattern handles initialization and
defines a name for the outer level `function'.
The following range pattern matches the function
header, and extracts from it the function name.
The third pattern records references to variables
which are not arguments, and thus, are global.
The final \fCEND\fP pattern handles the output.
.PP
Straightforward removal of unneeded newlines and
whitespaces could reduce the above code to just
five lines, but with a loss in readability.
.PP
While this example is somewhat more complex than
the one-liners for which
.BR awk (1)
is justly famous, it is surprising how little code
is needed to produce a perfectly-reliable result
for what would otherwise be a rather difficult job
in most other programming languages, and one
.I impossible
to do reliably with simple pattern matching on the
.BR awk (1)
program source text.
.PP
This example is so generally useful that it is
provided via the
.B \-\-globals
option in
.BR awkpretty (1).
.\"=====================================================================
.SH "SEE ALSO"
.BR awk (1),
.BR awkpretty (1),
.BR awkunlex (1),
.BR gawk (1),
.BR mawk (1),
.BR nawk (1).
.\"=====================================================================
.SH AUTHORS
The
.BR awk (1)
implementation on which
.B awklex
is based was written by, and is maintained by,
Brian Kernighan of Lucent Technologies (formerly,
AT&T) Bell Laboratories. The source code is freely
available on the World-Wide Web at:
.RS
\&\fChttp://cm.bell-labs.com/cm/cs/awkbook/index.html\fP
.RE
and the
.BR awk (1)
programming language is completely described in
the excellent book:
.RS
.nf
Alfred V. Aho, Brian W. Kernighan and Peter J. Weinberger
.I "The AWK Programming Language"
Addison-Wesley, 1988
ISBN 0-201-07981-X
.fi
.RE
.PP
The minor modifications (6 lines of changes, plus
about 165 new lines) of the original
.BR awk (1)
code (about 12,000 lines) to produce
.BR awklex ,
and this documentation, were done by:
.RS
.nf
Nelson H. F. Beebe
Center for Scientific Computing
University of Utah
Department of Mathematics, 322 INSCC
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
USA
Email: \fCbeebe@math.utah.edu\fP, \fCbeebe@acm.org\fP, \fCbeebe@ieee.org\fP
WWW URL: \fChttp://www.math.utah.edu/~beebe\fP
Telephone: +1 801 581 5254
FAX: +1 801 585 1640, +1 801 581 4148
.fi
.RE
It is a tribute to the great skill and care with
which
.BR awk (1)
has been implemented that
.B awklex
could be created with such tiny changes.
.\"=====================================================================
.SH COPYRIGHT
Almost all of the code in
.B awklex
is derived from the
.BR awk (1)
distribution, for which a copyright applies, and
as required by that copyright, it is reproduced
verbatim here in this documentation:
.RS
.nf
/****************************************************************
Copyright (C) Lucent Technologies 1997
All Rights Reserved

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name Lucent Technologies or any of
its entities not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.

LUCENT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
IN NO EVENT SHALL LUCENT OR ANY OF ITS ENTITIES BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
THIS SOFTWARE.
****************************************************************/
.fi
.RE
.\"=====================================================================
.\" This is for GNU Emacs file-specific customization:
.\" Local Variables:
.\" fill-column: 50
.\" End:
